import gpiozero
import blinker


class MotionDetector:

    start = blinker.Signal()
    end = blinker.Signal()

    def __init__(self, pin):
        self._sensor = gpiozero.MotionSensor(pin)

    def run(self):
        while True:
            self._sensor.wait_for_motion()
            self.start.send(self)

            self._sensor.wait_for_no_motion()
            self.end.send(self)



import RPi.GPIO as GPIO


class Led:
    def __init__(self, led_pin, initial_value=False):
        self._led_pin = led_pin
        GPIO.setup(self._led_pin, GPIO.OUT)
        GPIO.output(self._led_pin, initial_value)

    def on(self):
        GPIO.output(self._led_pin, True)

    def off(self):
        GPIO.output(self._led_pin, False)



import telegram


class Telegram:

    def __init__(self, token, recipient_id):
        self._recipient_id = recipient_id
        self._bot = telegram.Bot(token)

    def send_message(self, msg):
        self._bot.send_message(chat_id=self._recipient_id, text=msg)

    def send_doc(self, path):
        with open(path, 'rb') as doc:
            self._bot.send_document(chat_id=self._recipient_id, document=doc)



import datetime

import picamera
import blinker


class CamRecorder:

    start = blinker.Signal()
    end = blinker.Signal()

    def __init__(self):
        self._camera = picamera.PiCamera()
        self._camera.vflip = True
        self._camera.hflip = True

    def record(self):
        self._filename = datetime.datetime.now().strftime("%Y-%m-%d_%H.%M.%S.h264")
        self._camera.start_recording(self._filename)

        self.start.send(self, filename=self._filename)

    def stop(self):
        self._camera.stop_recording()

        self.end.send(self, filename=self._filename)



import RPi.GPIO as GPIO
import yaml
import argparse
import threading

from metis import motion, camera, led, telegram


def parse_command_line():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-c', '--conf', default='config.yaml',
        help='set path to the config file')

    arguments = parser.parse_args()

    return arguments

if __name__ == '__main__':
    with open(parse_command_line().conf, 'r') as stream:
        config = yaml.load(stream)

    GPIO.setmode(GPIO.BCM)

    detector = motion.MotionDetector(config['pins']['motion_sensor'])
    recorder = camera.CamRecorder()
    motion_led = led.Led(config['pins']['led_motion_on'])
    no_motion_led = led.Led(config['pins']['led_motion_off'], True)

    detector.start.connect(lambda sender: motion_led.on(), weak=False)
    detector.start.connect(lambda sender: no_motion_led.off(), weak=False)
    detector.start.connect(lambda sender: recorder.record(), weak=False)

    detector.end.connect(lambda sender: motion_led.off(), weak=False)
    detector.end.connect(lambda sender: no_motion_led.on(), weak=False)
    detector.end.connect(lambda sender: recorder.stop(), weak=False)

    recorder.end.connect(lambda sender, filename: print(filename, 'recorded'), weak=False)

    if 'telegram' in config:
        telega = telegram.Telegram(config['telegram']['token'], config['telegram']['recipient'])
        detector.start.connect(lambda sender: telega.send_message('Intruder detected'), weak=False)
        recorder.end.connect(
           lambda sender, filename: \
               threading.Thread(target=telega.send_doc, args=(filename,)).start(),
           weak=False)

    try:
        detector.run()
    except (KeyboardInterrupt, SystemExit):
        print('Exiting')
    finally:
        motion_led.off()
        no_motion_led.off()
        GPIO.cleanup()



import os

from io import open
from setuptools import setup, find_packages


here = os.path.dirname(__file__)

with open(os.path.join(here, 'README.rst'), 'r', encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='metispi',
    description='In brightest day in blackest night no evil shall escape your sight!',
    long_description=long_description,
    license='MIT',
    url='https://github.com/olha-kurkaiedova/metis',
    keywords='RaspberryPi RPi motion sensor',
    author='Olha Kurkaiedova',
    author_email='olya.kurkaedova@gmail.com',
    packages=find_packages(exclude=['docs', 'tests*']),
    use_scm_version=True,
    setup_requires=[
        'setuptools_scm',
    ],
    install_requires=[
        'RPi.GPIO >= 0.6.3',
        'gpiozero >= 1.3.2',
        'picamera >= 1.13',
        'blinker >= 1.4',
        'python-telegram-bot >= 6.1.0',
        'pyyaml >= 3.12',
    ],
)
